#!/usr/bin/env python
import os
import cgi
import shutil
import subprocess
import sys
import re
from optparse import OptionParser

DCF_HEADER="""
<!DOCTYPE DCF>
<DCF ref="index.html" title="CMake Reference Documentation">
"""

DCF_FOOTER="""
</DCF>
"""

HTML_HEADER="""
<html>
<head>
  <title>%(title)s</title>
  <link href="classic.css" rel="stylesheet" type="text/css" />
</head>
<body>
<table border="0" cellpadding="0" cellspacing="0" width="100%%">
  <tr>
    <td class="postheader">
      CMake Documentation
      &middot;
      <a href='index.html'><font color="#004FAF">Index</font></a>
      &middot;
      <a href='commands.html'><font color="#004FAF">Commands</font></a>
      &middot;
      <a href='modules.html'><font color="#004FAF">Modules</font></a>
    </td>
  </tr>
</table>
<h1 align="center">%(title)s</h1>
"""

HTML_FOOTER="""
</body>
</html>
"""
def linkKeywords(line, keywords):
    for keyword in keywords:
        htmlFile = keyword.lower() + ".html"
        link = "<a href='%s'>%s</a>" % (htmlFile, keyword)
        line = re.sub("(\W)" + keyword + "(\W)", "\\1" + link + "\\2", line)
    return line


class CMakeDocReader(object):
    __slots__ = ['title', 'type']

    def __init__(self, title, type):
        self.title = title
        self.type = type

    def getItemList(self):
        option = "--help-" + self.type + "-list"
        proc = subprocess.Popen(["cmake", option], stdout=subprocess.PIPE)
        # Skip cmake version
        proc.stdout.readline()

        return [x.strip() for x in proc.stdout.readlines()]

    def getItemDoc(self, command):
        option = "--help-" + self.type
        proc = subprocess.Popen(["cmake", option, command], stdout=subprocess.PIPE)
        # Skip cmake version
        proc.stdout.readline()
        # Skip command itself
        proc.stdout.readline()

        return proc.stdout.read()


def formatDoc(command, doc, keywords):
    # Remove ourself from the keyword list, but do not alterate the list passed
    # by caller
    keywords = keywords[:]
    if command in keywords:
        keywords.remove(command)

    html = [ HTML_HEADER % dict(title=command) ]

    mode = "none"
    lines = doc.split("\n")
    lines.reverse()
    while len(lines) > 0:
        line = lines.pop()
        line = line[7:]
        line = cgi.escape(line)
        if mode == "none":
            if line.strip() == "":
                continue
            if line[0] != " ":
                mode = "para"
                html.append("<p>")
            else:
                mode = "code"
                html.append("<pre>")

        if mode == "para":
            if line.strip() == "":
                html.append("</p>\n")
                mode = "none"
            else:
                line = linkKeywords(line, keywords)
                html.append(line + " ")

        elif mode == "code":
            if line.strip() == "":
                # eat back last "\n"
                tmp = html.pop()
                assert tmp == "\n"
                html.append("</pre>\n")
                mode = "none"
            else:
                html.append(line)
                html.append("\n")

    html.append(HTML_FOOTER)

    return "".join(html)


def writeSectionBegin(dcf, docReader, outputDir):
    htmlFile = docReader.title.lower() + ".html"
    print >>dcf, "<section ref='%s' title='%s'>" % (htmlFile, docReader.title)

    # Create section HTML file
    fl = file(os.path.join(outputDir, htmlFile), "w")
    print >>fl, HTML_HEADER % dict(title=docReader.title)
    print >>fl, "<ul>"
    for item in docReader.getItemList():
        htmlFile = item.lower() + ".html"
        print >>fl, "<li><a href='%s'>%s</a></li>" % (htmlFile, item)
    print >>fl, "</ul>"
    print >>fl, HTML_FOOTER


def writeSectionEnd(dcf):
    print >>dcf, "</section>"


def writeKeyword(dcf, keyword):
    htmlFile = keyword.lower() + ".html"
    print >>dcf, "<keyword ref='%s'>%s</keyword>" % (htmlFile, keyword)


def generateDoc(docReader, dcf, outputDir):
    items = docReader.getItemList()
    for item in items:
        print item
        doc = docReader.getItemDoc(item)
        html = formatDoc(item, doc, items)
        htmlFile = os.path.join(outputDir, item.lower() + ".html")
        file(htmlFile, "w").write(html)

    writeSectionBegin(dcf, docReader, outputDir)
    for item in items:
        writeKeyword(dcf, item)
    writeSectionEnd(dcf)


def generateIndex(readers, outputDir):
    indexFile = os.path.join(outputDir, "index.html")
    fl = file(indexFile, "w")
    header = HTML_HEADER % dict(title="CMake Documentation Index")
    print >>fl, header

    print >>fl, "<table cellpadding='2' cellspacing='1' border='0' width='100%' bgcolor='#e5e5e5'>"
    print >>fl, "<tr>"
    for reader in readers:
        print >>fl, "<th bgcolor='#a2c511' width='50%%'>%s</th>" % reader.title
    print >>fl, "</tr>"

    print >>fl, "<tr>"
    for reader in readers:
        print >>fl, "<td valign='top'>"
        print >>fl, "<ul>"
        for item in reader.getItemList():
            htmlFile = item.lower() + ".html"
            print >>fl, "<li><a href='%s'>%s</a></li>" % (htmlFile, item)
        print >>fl, "</ul>"
        print >>fl, "</td>"
    print >>fl, "</tr>"

    print >>fl, "</table>"
    print >>fl, HTML_FOOTER


def prepareOutputDir(outputDir):
    if not os.path.exists(outputDir):
        os.makedirs(outputDir)

    assert os.path.isdir(outputDir)

    shutil.copy("classic.css", outputDir)


def main():
    parser = OptionParser()

    # Add an option which takes an argument and is stored in options.filename.
    # 'metavar' is an example of argument and should match the text in 'help'.
    parser.add_option("-o", "--output", dest="outputDir", default="cmake",
                      help="dir wher the doc will be generated")

    (options, args) = parser.parse_args()

    prepareOutputDir(options.outputDir)

    dcfFile = os.path.join(options.outputDir, "cmake.dcf")
    dcf = file(dcfFile, "w")
    dcf.write(DCF_HEADER)
    commandDocReader = CMakeDocReader("Commands", "command")
    modulesDocReader = CMakeDocReader("Modules", "module")
    generateIndex([commandDocReader, modulesDocReader], options.outputDir)
    generateDoc(commandDocReader, dcf, options.outputDir)
    generateDoc(modulesDocReader, dcf, options.outputDir)
    dcf.write(DCF_FOOTER)

    return 0


if __name__=="__main__":
    sys.exit(main())
# vi: ts=4 sw=4 et
